<?php
namespace Ak\Wp\Init;

use Ak\Wp\Plugin\PluginManager;
use Psr\Container\ContainerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Laminas\ServiceManager\PluginManagerInterface;
use Laminas\Stratigility\MiddlewarePipe;
use Laminas\Stratigility\MiddlewarePipeInterface;

/**
 * Omogućava vanjskim komponentama dodavanje middlewareova u glavni pipe.
 *
 * @package Ak\Wp\Init
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/wp
 */
final class Middleware implements MiddlewarePipeInterface
{
    /** @var MiddlewarePipeInterface */
    private $pipe;

    /**
     * Konstruktor.
     *
     * @param ContainerInterface $container
     */
    public function __construct(ContainerInterface $container)
    {
        $this->pipe = new MiddlewarePipe();

        $config = $container->get('config')['wp'] ?? [];
        $middlewares = apply_filters('ak_wp_middlewares', $config['middleware'] ?? []);

        foreach ($middlewares as $middleware) {
            if (is_string($middleware)) {
                if ($container->has($middleware)) {
                    $middleware = $container->get($middleware);
                } else {
                    /** @var PluginManagerInterface $plugins */
                    $plugins = $container->get(PluginManager::class);
                    $middleware = $plugins->get($middleware);
                }
            }

            $this->pipe->pipe($middleware);
        }
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        return $this->pipe->process($request, $handler);
    }

    public function pipe(MiddlewareInterface $middleware): void
    {
        $this->pipe->pipe($middleware);
    }

    /**
     * Handles a request and produces a response.
     *
     * May call other collaborating code to generate the response.
     *
     * @param ServerRequestInterface $request
     * @return ResponseInterface
     */
    public function handle(ServerRequestInterface $request): ResponseInterface
    {
        return $this->pipe->handle($request);
    }
}
